/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ForecastFox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Jon Stritar <jstritar@MIT.EDU>
 *   Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//global forecastfox object
var gForecastfox = null;

function wfLoad()
{
  gForecastfox = new Forecastfox();
  gForecastfox.init();
}

function wfUnload()
{
  gForecastfox.destroy();
  gForecastfox = null;
}

///////////////////////
// Forecastfox Object
function Forecastfox()
{
  return this;
}

Forecastfox.prototype =
{
  mUpdate: null,
  mDomain: "forecastfox.",
  mBranch: null,
  mProfileLoading: false,
  mIsMoving: false,
  
  init: function()
  {
    //get our XPCOM update service
    this.mUpdate = Components.classes["@mozdev.org/forecastfox/update;1"]
                             .getService(Components.interfaces.nsIForecastfoxUpdate);
    this.mProfile = Components.classes["@mozdev.org/forecastfox/profile;1"]
              .getService(Components.interfaces.nsIForecastfoxProfile);
    //set deck
    var deck = document.getElementById("wf_deck");
    deck.setAttribute("selectedIndex", "2");

    //setup preference service
    var prefService = "@mozilla.org/preferences-service;1";
    prefService = Components.classes[prefService].getService(Components.interfaces.nsIPrefService);
    this.mBranch = prefService.getBranch(this.mDomain);

    //add preference observer
    var pbi = this.mBranch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    pbi.addObserver("", this, false);

    //add observer to nsForecastfox:update
    var os = Components.classes["@mozilla.org/observer-service;1"]
                       .getService(Components.interfaces.nsIObserverService);
    os.addObserver(this, "forecastfox:update:start", false);
    os.addObserver(this, "forecastfox:update:error", false);
    os.addObserver(this, "forecastfox:update:end:current", false);
    os.addObserver(this, "forecastfox:update:end:forecast", false);
    os.addObserver(this, "forecastfox:update:end:links", false);
    os.addObserver(this, "forecastfox:profile:start", false);
    os.addObserver(this, "forecastfox:profile:end", false);
    os.addObserver(this, "forecastfox:profile:error", false);

    //move it to the correct place
    this.moveBox();

    //setup display - give window a chance to display before loading
    if (this.mUpdate.firstRun)
      setTimeout("gForecastfox.mUpdate.load(false);", 50);
    else
      this.updateAll();
  },

  destroy: function()
  {
    try {
      //remove preference observer
      var pbi = this.mBranch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
      pbi.removeObserver("", this);
    } catch(e) {}

    try {
      //remove observers
      var os = Components.classes["@mozilla.org/observer-service;1"]
                         .getService(Components.interfaces.nsIObserverService);
      os.removeObserver(this, "forecastfox:update:start");
      os.removeObserver(this, "forecastfox:update:error");
      os.removeObserver(this, "forecastfox:update:end:current", false);
      os.removeObserver(this, "forecastfox:update:end:forecast", false);
      os.removeObserver(this, "forecastfox:update:end:links", false);
      os.removeObserver(this, "forecastfox:profile:start", false);
      os.removeObserver(this, "forecastfox:profile:end", false);
      os.removeObserver(this, "forecastfox:profile:error", false);
    } catch(e) {}

    //destroy variables
    this.mUpdate = null;
    this.mDomain = null;
    this.mBranch = null;
    this.mProfileLoading = null;
  },

  observe: function(aSubject, aTopic, aData)
  {
    switch (aTopic) {
      case "forecastfox:update:start":
        this.updateStart();
        break;
      case "forecastfox:update:error":
        this.updateError();
        break;
      case "forecastfox:update:end:current":
        this.updateCurrent();
        break;
      case "forecastfox:update:end:forecast":
        this.updateForecast();
        break;
      case "forecastfox:update:end:links":
        this.updateLinks();
        break;
      case "forecastfox:profile:start":
        this.mProfileLoading = true;
        break;
      case "forecastfox:profile:end":
        this.mProfileLoading = false;
        this.moveBox();
        break;
      case "forecastfox:profile:error":
        this.mProfileLoading = false;
        break;
      case "nsPref:changed":
        switch (aData) {
          case "current.tooltip":
          case "current.label":
          case "current.mode":
          case "current.tipicon":
            if (!this.mProfileLoading)
              this.updateCurrent();
            break;
          case "forecast.tooltip":
          case "forecast.label":
          case "forecast.mode":
          case "forecast.parts":
          case "forecast.tipicon":
            if (!this.mProfileLoading)
              this.updateForecast();
            break;
          case "current.display":
          case "forecast.days":
            if (!this.mProfileLoading)
              this.updateAll();
            break;
          case "display.bar" :
          case "display.nextnode" :
          case "display.insertafter" :
            if (!this.mProfileLoading && !this.mIsMoving)
              this.moveBox();
            break;
        }
    }
  },

  moveBox: function()
  {
    //check if box is in the right location
    var toolbar   = document.getElementById(this.mBranch.getCharPref("display.bar"));
    var nextnode  = document.getElementById(this.mBranch.getCharPref("display.nextnode"));
    var insertafter = this.mBranch.getBoolPref("display.insertafter");
    var wdeck     = document.getElementById("wf_deck");

    //set that we are moving
    this.mIsMoving = true;
    
    //if nextnode doesn't exist move to default location
    if (!nextnode) {
      this.mBranch.setCharPref("display.bar", "status-bar");
      this.mBranch.setBoolPref("display.insertafter", true); 
      this.mIsMoving = false;               
      this.mBranch.setCharPref("display.nextnode", "livemark-button");
      return;
    }
    
    // bail if it is in the right place or the prefs are not in sync
    var neighbor = ((insertafter) ? nextnode.nextSibling : nextnode.previousSibling);
    if (neighbor)
      neighbor = neighbor.getAttribute("id");

    if (((neighbor == "wf_deck") && (wdeck.parentNode == toolbar)) ||
        (nextnode.parentNode != toolbar)) {
      this.mIsMoving = false; 
      return;
    }
  
    //remove us from parent
    wdeck.parentNode.removeChild(wdeck);

    //place box
    if (insertafter) {
      var next_node = nextnode.nextSibling;
      if (next_node)
        toolbar.insertBefore(wdeck,next_node);
      else
        toolbar.appendChild(wdeck);
    }
    else
      toolbar.insertBefore(wdeck,nextnode);
      
    this.mIsMoving = false;
  },

  updateAll: function()
  {
    if (this.mUpdate.getValue("lastError","type",0) != "") {
      this.updateCurrent();
      this.updateForecast();
      this.updateLinks();
    }
    else
      this.updateError();
  },

  updateStart: function()
  {
    var wdeck     = document.getElementById("wf_deck");
    var wprogress = document.getElementById("wf_progress");
    var werror    = document.getElementById("wf_error");
    var wstrings  = document.getElementById("wf_bundle");

    //remove error
    werror.removeAttribute("value");
    werror.removeAttribute("tooltiptext");
    werror.setAttribute("collapsed", "true");

    //set progress meter
    wprogress.setAttribute("mode", "undetermined");
    wprogress.setAttribute("tooltiptext", wstrings.getString("wf.tooltip.start"));
    wprogress.removeAttribute("value");
    wprogress.removeAttribute("collapsed");

    //show meter
    wdeck.setAttribute("selectedIndex", "1");
  },

  updateError: function()
  {
    var wdeck     = document.getElementById("wf_deck");
    var wprogress = document.getElementById("wf_progress");
    var werror    = document.getElementById("wf_error");

    //remove progress
    wprogress.setAttribute("mode", "determined");
    wprogress.setAttribute("value", "0");
    wprogress.removeAttribute("tooltiptext");
    wprogress.setAttribute("collapsed", "true");

    //set error
    werror.setAttribute("value", this.mUpdate.getValue("lastError", "msg", 0));
    werror.setAttribute("tooltiptext", this.mUpdate.getValue("lastError", "tip", 0));    
    werror.removeAttribute("collapsed");

    //show error
    wdeck.setAttribute("selectedIndex", "2");
  },
  
  updateCurrent: function()
  {
    var wdeck     = document.getElementById("wf_deck");
    var wprogress = document.getElementById("wf_progress");
    var werror    = document.getElementById("wf_error");
    var wstrings  = document.getElementById("wf_bundle");
    var wcurrent  = document.getElementById("wf_current");
    var wspacer   = document.getElementById("wf_spacer");
    var tooltip;
  
    //remove progress
    wprogress.setAttribute("mode", "determined");
    wprogress.setAttribute("value", "0");
    wprogress.removeAttribute("tooltiptext");
    wprogress.setAttribute("collapsed", "true");

    //remove error
    werror.removeAttribute("value");
    werror.removeAttribute("tooltiptext");    
    werror.setAttribute("collapsed", "true");

    //if both forecast and current is shown, uncollapse the spacer
    if ((this.mBranch.getIntPref("forecast.days") > 0) && (this.mBranch.getBoolPref("current.display")))
      wspacer.removeAttribute("collapsed");
    else
      wspacer.setAttribute("collapsed", true);

    //only update is showing
    if (!this.mBranch.getBoolPref("current.display")) {
      wcurrent.setAttribute("collapsed", "true");
      return
    }

  //if tooltip has been set before remove it
    if (wcurrent.hasAttribute("tooltip")) {
      tooltip = document.getElementById(wcurrent.getAttribute("tooltip"));
      tooltip.parentNode.removeChild(tooltip);
      wcurrent.removeAttribute("tooltip");
    }
      
    wcurrent.removeAttribute("collapsed");

    //set generic loc attributes
    count = {};
    keys = this.mUpdate.getKeys("location", count);
    for (y=0; y < count.value; y++)
      wcurrent.setAttribute(keys[y], this.mUpdate.getValue("location", keys[y], 0));

    //get all attributes for current day
    var count = {};
    var keys = this.mUpdate.getKeys("current", count);
    for (x=0; x < count.value; x++)
      wcurrent.setAttribute(keys[x], this.mUpdate.getValue("current", keys[x], 0));

    //no info for today
    if (wcurrent.getAttribute("icon") == "-") {
      wcurrent.setAttribute("src", "chrome://forecastfox/skin/images/na.png");
      wcurrent.setAttribute("tooltiptext", wstrings.getString("wf.tooltip.nodata"));
    }
    else {
      this.createTooltip(wcurrent, "CURRENT");
      wcurrent.removeAttribute("tooltiptext");        
      switch (this.mBranch.getIntPref("current.mode")) {
        case 0:
        default:
          wcurrent.setAttribute("src", "chrome://forecastfox/skin/images/small/" + wcurrent.getAttribute("icon") + ".png");
          wcurrent.removeAttribute("label");
          break;
        case 1:
          wcurrent.removeAttribute("src");
          if (this.mBranch.prefHasUserValue("current.label"))
            wcurrent.setAttribute("label", this.setLabel(this.mBranch.getCharPref("current.label"), "CURRENT", wcurrent));
          else
            wcurrent.setAttribute("label", this.setLabel(wstrings.getString("wf.cc.label"), "CURRENT", wcurrent));          
          break;
        case 2:
          wcurrent.setAttribute("src", "chrome://forecastfox/skin/images/small/" + wcurrent.getAttribute("icon") + ".png");
          if (this.mBranch.prefHasUserValue("current.label"))
            wcurrent.setAttribute("label", this.setLabel(this.mBranch.getCharPref("current.label"), "CURRENT", wcurrent));
          else
            wcurrent.setAttribute("label", this.setLabel(wstrings.getString("wf.cc.label"), "CURRENT", wcurrent));   
          break;
      }
    }
    //show current
    wdeck.setAttribute("selectedIndex", "0");
  },

  updateForecast: function()
  {
    var wdeck     = document.getElementById("wf_deck");
    var wprogress = document.getElementById("wf_progress");
    var werror    = document.getElementById("wf_error");
    var wstrings  = document.getElementById("wf_bundle");
    var wforecast = document.getElementById("wf_forecast");

    //remove progress
    wprogress.setAttribute("mode", "determined");
    wprogress.setAttribute("value", "0");
    wprogress.removeAttribute("tooltiptext");
    wprogress.setAttribute("collapsed", "true");

    //remove error
    werror.removeAttribute("value");
    werror.removeAttribute("tooltiptext");    
    werror.setAttribute("collapsed", "true");

    //remove old forecast
    while (wforecast.hasChildNodes()) {
      if (wforecast.lastChild.hasAttribute("tooltip")) {
        tooltip = document.getElementById(wforecast.lastChild.getAttribute("tooltip"));
        tooltip.parentNode.removeChild(tooltip);
        wforecast.lastChild.removeAttribute("tooltip");
      }    
      wforecast.removeChild(wforecast.lastChild);
    }
    
    //hide the box and return if we don't show any days
    if (this.mBranch.getIntPref("forecast.days") == 0) {
      wforecast.setAttribute("collapsed", true);
      return;
    }

    wforecast.removeAttribute("collapsed");

    var x, y, node, part, keys;
    var cDate   = new Date();
    var cHours    = cDate.getHours();
    var cYear   = cDate.getFullYear();
    var day_index   = 0;
    var count     = {};
    var mode    = this.mBranch.getIntPref("forecast.parts");
    var days    = this.mBranch.getIntPref("forecast.days")
    var tooltip;
    
    if (mode == 2)
      days = days * 2;

    for (x=0; x < days; x++) {
      node = document.createElement("statusbarpanel");
  
      //set context menu on node
      node.setAttribute("context","wf_popup");
       
      //if there is no data, go to the next day
      //or if the data is old (fixes bug caused by server not changing to next day until morning)
      if (((this.mUpdate.getValue("day", "icon", day_index) == "-") && (cHours < 15)) ||
         (new Date(this.mUpdate.getValue("forecast", "date", day_index) + ", " + cYear) < new Date(cYear, cDate.getMonth(), cDate.getDate())))
        day_index++;

      //set generic loc attributes
      count = {};
      keys = this.mUpdate.getKeys("location", count);
      for (y=0; y < count.value; y++)
        node.setAttribute(keys[y], this.mUpdate.getValue("location", keys[y], day_index));

      //set generic forecast attributes
      count = {};
      keys = this.mUpdate.getKeys("forecast", count);
      for (y=0; y < count.value; y++)
        node.setAttribute(keys[y], this.mUpdate.getValue("forecast", keys[y], day_index));

      // set night/day specific attributes
      if ((mode == 1) ||
         ((mode == 2) && (((x % 2 == 1) && (cHours < 15)) || ((x % 2 == 0) && (cHours >= 15)))) ||
         ((x == 0) && (cHours >= 15))) {

        // Set the night attributes if it is in mode 1 (all nights)
        // or if its mode two and an even entry (day, night, day, night ... etc)
        // or if its today's forecast and is after 2pm
        part = "night";

        //set night attributes
        count = {};
        keys = this.mUpdate.getKeys("night", count);
        for (y=0; y < count.value; y++)
          node.setAttribute(keys[y], this.mUpdate.getValue("night", keys[y], day_index));
      }
      else {
        part = "day";

        //set day attributes
        count = {};
        keys = this.mUpdate.getKeys("day", count);
        for (y=0; y < count.value; y++)
          node.setAttribute(keys[y], this.mUpdate.getValue("day", keys[y], day_index));
      }

      // set the image, class and tooltip
      node.setAttribute("part", part);
      node.setAttribute("id", "forecast" + part + day_index);
      node.setAttribute("class", "forecastfox-panel");
      this.createTooltip(node, "FORECAST");
      switch (this.mBranch.getIntPref("forecast.mode")) {
        case 0:
        default:
          node.setAttribute("src", "chrome://forecastfox/skin/images/small/" + node.getAttribute("icon") + ".png");
          node.removeAttribute("label");
          break;
        case 1:
          node.removeAttribute("src");
          if (this.mBranch.prefHasUserValue("forecast.label"))
            node.setAttribute("label", this.setLabel(this.mBranch.getCharPref("forecast.label"), "FORECAST", node));
          else
            node.setAttribute("label", this.setLabel(wstrings.getString("wf.dayf.label"), "FORECAST", node)); 
          break;
        case 2:
          node.setAttribute("src", "chrome://forecastfox/skin/images/small/" + node.getAttribute("icon") + ".png");
          if (this.mBranch.prefHasUserValue("forecast.label"))
            node.setAttribute("label", this.setLabel(this.mBranch.getCharPref("forecast.label"), "FORECAST", node));
          else
            node.setAttribute("label", this.setLabel(wstrings.getString("wf.dayf.label"), "FORECAST", node)); 
          break;
      }

      wforecast.appendChild(node);

      if (mode == 2) {
        if (((x % 2 == 1) && (cHours < 15)) || 
             ((x % 2 == 0) && (cHours >= 15)))
          day_index++;
      }
      else
        day_index++;
    }

    wdeck.setAttribute("selectedIndex", "0");
  },

  updateLinks: function()
  {
    var popup = document.getElementById("twclinkspopup");
    var menu  = document.getElementById("twclinksmenu");

    //remove old links
    while (popup.hasChildNodes())
      popup.removeChild(popup.lastChild);

    //add links
    for (var x=0; x< this.mUpdate.getLinkCount(); x++) {
      var menuitem  = document.createElement("menuitem");
      menuitem.setAttribute("label", this.mUpdate.getValue("links","title",x));
      menuitem.setAttribute("url", this.mUpdate.getValue("links","url",x));
      menuitem.setAttribute("oncommand","gForecastfox.openPromo(event);");
      menuitem.setAttribute("onclick","checkForMiddleClick(this, event);");
      popup.appendChild(menuitem);
    }

    //XXX keep this disabled
    //if (this.mUpdate.getLinkCount() > 0)
      //menu.removeAttribute("disabled");
  },

  createTooltip: function(node, mode) {
    var tooltip, parent, lblparent, lbls, lbl, display;
    var strings = document.getElementById("wf_bundle");
    
    //create empty tip
    parent = document.getElementById("mainPopupSet");
    tooltip = document.createElement("tooltip");
    tooltip.setAttribute("id", node.getAttribute("id") + "_tip");
    tooltip.setAttribute("noautohide", true);
    
  //set display mode
    if (mode == "CURRENT")
      display = this.mBranch.getBoolPref("current.tipicon");
    else
      display = this.mBranch.getBoolPref("forecast.tipicon");
      
    //set image      
    if (display) {
      var imgparent = document.createElement("vbox");
      var image = document.createElement("image");
      image.setAttribute("src", "chrome://forecastfox/skin/images/large/" + node.getAttribute("icon") + ".png");
      image.setAttribute("class", "tipicon");
      imgparent.appendChild(image);
      lblparent = document.createElement("vbox");
      tooltip.setAttribute("orient", "horizontal");
      tooltip.appendChild(imgparent);
      tooltip.appendChild(lblparent);
    }
    else {
      tooltip.setAttribute("orient", "vertical");
      lblparent = tooltip;
    }
    
    //split tooltip
    if (mode == "CURRENT") { 
      if (this.mBranch.prefHasUserValue("current.tooltip"))
        lbls = this.mBranch.getCharPref("current.tooltip");
      else   
        lbls = strings.getString("wf.cc.tooltip");
    }
    else {
      if (this.mBranch.prefHasUserValue("forecast.tooltip"))
        lbls = this.mBranch.getCharPref("forecast.tooltip");
      else   
        lbls = strings.getString("wf.dayf.tooltip");
    }
      
    lbls = lbls.split("/n");  
    for (var i = 0; i < lbls.length; ++i) {
      lbl = document.createElement("label");
      lbl.setAttribute("value", this.setLabel(lbls[i], mode, node));
      lblparent.appendChild(lbl);
    }
        
    //append new tooltip
    parent.appendChild(tooltip);
    node.setAttribute("tooltip", tooltip.getAttribute("id"));
  },
  
  populateProfiles: function()
  {
    var prof_menu = document.getElementById("wf_prof_popup");
    var childs = prof_menu.childNodes;
    var strings = document.getElementById("wf_bundle");
    
    // first remove all menuitems
    for (var x = 0; x < childs.length; x++)
      prof_menu.removeChild(childs[x]);
    
    var count = {};
    var keys = this.mProfile.getNames(count);
    var item;

    if (count.value == 0) {
      item = document.createElement("menuitem");
      item.setAttribute("label", strings.getString("wf.profile.none"));
      item.setAttribute("disabled", true);
      prof_menu.appendChild(item);
      return;
    }
    
    for (x = 0; x < count.value; x++) {
      item = document.createElement("menuitem");
      item.setAttribute("label", keys[x]);
      item.setAttribute("oncommand", "gForecastfox.switchProfile(this)"); 
      item.setAttribute("type", "radio");
      item.setAttribute("name", "profile");     
      if (keys[x] == this.mBranch.getCharPref("profile"))
        item.setAttribute("checked", "true");
      prof_menu.appendChild(item);
    }

    return;
  },
  
  switchProfile: function(item)
  {
    document.getElementById("wf_prof_popup").hidePopup();
    var new_profile = item.getAttribute("label");
    if (new_profile != this.mBranch.getCharPref("profile"))
      this.mBranch.setCharPref("profile", new_profile);
  },
  
  openPromo: function(aEvent)
  {
    var url = aEvent.currentTarget.getAttribute("url") + "&par=" + this.mUpdate.partnerId;
    openUILink(url, aEvent);
  },

  openLink: function(aEvent, aLoc)
  {
    var url = aEvent.currentTarget.getAttribute("url");
    if (aLoc) {
      var locid = this.mBranch.getCharPref("locid");
      url = url + locid + "?prod=" + this.mUpdate.productCode + "&par=" + this.mUpdate.partnerId;
    }
    else
      url = url + "?prod=" + this.mUpdate.productCode + "&par=" + this.mUpdate.partnerId;
      
    if (aEvent.target.parentNode == document.getElementById("wf_forecast"))
      url = url + "#tenday";

    openUILink(url, aEvent);
  },
  
  setLabel: function(text, type, element) 
  {
    if (!text)
      return "";

    var tool_base = [[/\[locale\]/g, "element.getAttribute('locale')"],
                     [/\[prof\]/g, "this.mBranch.getCharPref('profile')"],
                     [/\[ut\]/g, "element.getAttribute('ut')"],
                     [/\[ud\]/g, "element.getAttribute('ud')"],
                     [/\[us\]/g, "element.getAttribute('us')"],
                     [/\[up\]/g, "element.getAttribute('up')"],
                     [/\[ur\]/g, "element.getAttribute('ur')"],
                     [/\[dnam\]/g, "element.getAttribute('dnam')"],
                     [/\[tm\]/g, "element.getAttribute('tm')"],
                     [/\[lat\]/g, "element.getAttribute('lat')"],
                     [/\[lon\]/g, "element.getAttribute('lon')"],
                     [/\[sunr\]/g, "element.getAttribute('sunr')"],
                     [/\[suns\]/g, "element.getAttribute('suns')"],
                     [/\[zone\]/g, "element.getAttribute('zone')"],
                     [/\[deg\]/g, "String.fromCharCode(176)"]];
    var tool_append = null;

    if (type == "CURRENT") {
      tool_append = [[/\[lsup\]/g, "element.getAttribute('lsup')"],
                    [/\[obst\]/g, "element.getAttribute('obst')"],
                    [/\[tmp\]/g, "element.getAttribute('tmp')"],
                    [/\[flik\]/g, "element.getAttribute('flik')"],
                    [/\[t\]/g, "element.getAttribute('t')"],
                    [/\[icon\]/g, "element.getAttribute('icon')"],
                    [/\[barr\]/g, "element.getAttribute('barr')"],
                    [/\[bard\]/g, "element.getAttribute('bard')"],
                    [/\[winds\]/g, "element.getAttribute('winds')"],
                    [/\[windgust\]/g, "element.getAttribute('windgust')"],
                    [/\[windd\]/g, "element.getAttribute('windd')"],
                    [/\[windt\]/g, "element.getAttribute('windt')"],
                    [/\[hmid\]/g, "element.getAttribute('hmid')"],
                    [/\[vis\]/g, "element.getAttribute('vis')"],
                    [/\[uvi\]/g, "element.getAttribute('uvi')"],
                    [/\[uvt\]/g, "element.getAttribute('uvt')"],
                    [/\[dewp\]/g, "element.getAttribute('dewp')"]];
    }
    else if (type == "FORECAST") {
      tool_append = [[/\[part\]/g, "element.getAttribute('part')"],
                    [/\[day\]/g, "element.getAttribute('day')"],
                    [/\[date\]/g, "element.getAttribute('date')"],
                    [/\[lsup\]/g, "element.getAttribute('lsup')"],
                    [/\[hi\]/g, "element.getAttribute('hi')"],
                    [/\[low\]/g, "element.getAttribute('low')"],
                    [/\[sunr\]/g, "element.getAttribute('sunr')"],
                    [/\[suns\]/g, "element.getAttribute('suns')"],
                    [/\[icon\]/g, "element.getAttribute('icon')"],
                    [/\[t\]/g, "element.getAttribute('t')"],
                    [/\[winds\]/g, "element.getAttribute('winds')"],
                    [/\[windgust\]/g, "element.getAttribute('windgust')"],
                    [/\[windd\]/g, "element.getAttribute('windd')"],
                    [/\[windt\]/g, "element.getAttribute('windt')"],
                    [/\[ppcp\]/g, "element.getAttribute('ppcp')"],
                    [/\[hmid\]/g, "element.getAttribute('hmid')"],
                    [/\[sday\]/g, "element.getAttribute('day').substr(0,3)" ],
                    [/\[tmp\]/g, "(element.getAttribute('part') == 'day') ? element.getAttribute('hi') : element.getAttribute('low')"]];
    }
    var tool_array = concat(tool_base, tool_append);
    
    // Then replace each one with the value.
    for (var x = 0; x < tool_array.length; x++)
        text = text.replace(tool_array[x][0], eval(tool_array[x][1]));

    return text;
  },
  
  checkForMiddleClick:function (node, event)
  {
    if (event.button == 1) {
      eval(node.getAttribute("ondblclick"));
      closeMenus(event.target);
    }
  }
}


function concat(c1, c2)
{
  // Concats too collections into an array.
  var c3 = new Array(c1.length + c2.length);
  var x,y = 0;

  for (x = 0; x < c1.length; x++)
    c3[y++] = c1[x];

  for (x = 0; x < c2.length; x++)
    c3[y++] = c2[x];

  return c3;
}
