/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ForecastFox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Jon Stritar <jstritar@MIT.EDU>
 *   Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var wfOptions = null;

function wfOptionsLoad() 
{
  wfOptions = new wfOptionsManager();
  wfOptions.init();
}

function wfOptionsUnload()
{
  wfOptions.destroy();
  wfOptions = null;
}

function wfOptionsManager()
{
  return this;
}

wfOptionsManager.prototype =
{
  init: function()
  {
    this.wfUpdate = Components.classes["@mozdev.org/forecastfox/update;1"]
                       .getService(Components.interfaces.nsIForecastfoxUpdate);
    this.wfProfile = Components.classes["@mozdev.org/forecastfox/profile;1"]
              .getService(Components.interfaces.nsIForecastfoxProfile);
              
    //get preference branch
    var prefService = "@mozilla.org/preferences-service;1";
    prefService = Components.classes[prefService].getService(Components.interfaces.nsIPrefService); 
    this.wfBranch = prefService.getBranch("forecastfox.");
    
    // add profile observer
    this.wfObs = Components.classes["@mozilla.org/observer-service;1"]
                       .getService(Components.interfaces.nsIObserverService);
                       
    this.wfObs.addObserver(this, "forecastfox:profile:new", false);
    this.wfObs.addObserver(this, "forecastfox:profile:start", false);
    this.wfObs.addObserver(this, "forecastfox:profile:end", false);
  
    //load previous preferences
    this.updatePrefs();
    
    this.locid = document.getElementById("locidinput");
    this.cc_display = document.getElementById("wfdisplaycc");
    this.cc_alert = document.getElementById("alertinput");
    this.cc_temp = document.getElementById("wfcctemp");
    this.cc_img = document.getElementById("wfccimage");
    this.cc_alert_disable = document.getElementById("wf_alert_disable");
    this.metric = document.getElementById("tempgroup");
    this.temp_c = document.getElementById("celcius");
    this.temp_f = document.getElementById("fahrenheit");
    this.barinput = document.getElementById("barinput");
    this.nextnode = document.getElementById("neighborinput");
    this.align = document.getElementById("aligninput");
    this.dayf_days = document.getElementById("daysinput");
    this.dayf_show_days = document.getElementById("wfdays");
    this.dayf_show_nights = document.getElementById("wfnights");
    this.dayf_show_temp = document.getElementById("wfdayftemp");
    this.dayf_show_img = document.getElementById("wfdayfimage");
    this.profile_list = document.getElementById("wf_prof_list");
    this.profile_current = document.getElementById("wf_prof_currentval");
    this.button_accept = document.documentElement.getButton("accept");
    this.button_apply = document.documentElement.getButton("help");
    this.button_remove = document.getElementById("wf_prof_delete");
    this.button_select = document.getElementById("wf_prof_select");
    this.profile_inuse = document.getElementById("wf_prof_inuse");
    this.button_rename = document.getElementById("wf_prof_rename");

    
    // Set up the display for the prefs
    this.syncPrefs(this.wfPrefs);
    this.populateBars();
    this.syncPref("display.bar", this.wfPrefs["display.bar"]);
    this.populateNodes();
    this.syncPref("display.nextnode", this.wfPrefs["display.nextnode"]);
    this.populateProfiles();
    
    this.profile_current.setAttribute("disabled", true);
    this.button_accept.setAttribute("disabled", true);
    this.button_apply.setAttribute("disabled", true);
    this.button_apply.label = document.getElementById("extra1-label").getAttribute("label");
    this.button_apply.removeAttribute("accesskey");

    //see if alerts are available
    try {
      var alerts = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);    
      document.getElementById("wf_alertbox").removeAttribute("collapsed");
    }
    catch(e) {
      document.getElementById("wf_alertbox").setAttribute("collapsed", "true");    
    }
    
  },
  
  destroy: function()
  {
    // remove observers
    this.wfObs.removeObserver(this, "forecastfox:profile:new", false);
    this.wfObs.removeObserver(this, "forecastfox:profile:start", false);
    this.wfObs.removeObserver(this, "forecastfox:profile:end", false);
    
    // then remove all references to variables
    this.wfUpdate = null;
    this.wfProfile = null;
    this.wfBranch = null;
    this.wfObs = null;
    this.wfPrefs = null;
    this.locid = null;
    this.cc_display = null;
    this.cc_alert = null;
    this.cc_temp = null;
    this.cc_img = null;
    this.metric = null;
    this.temp_c = null;
    this.temp_f = null;
    this.barinput = null;
    this.nextnode = null;
    this.align = null;
    this.dayf_days = null;
    this.dayf_show_days = null;
    this.dayf_show_nights = null;
    this.dayf_show_temp = null;
    this.dayf_show_img = null;
    this.profile_list = null;
    this.profile_current = null;
    this.button_accept = null;
    this.button_apply = null;
    this.profile_inuse = null;
    this.cc_alert_disable = null;
  },
  
  syncPrefs: function(prefs)
  {
    for (var value in prefs)
      this.syncPref(value, prefs[value]);
  },
  
  syncPref: function(pref, value)
  {
    switch(pref) {
      case "current.display":
        if (value)
          this.cc_display.setAttribute("checked", "true");
        else {
          this.cc_display.removeAttribute("checked");
          this.cc_img.setAttribute("disabled", true);
          this.cc_temp.setAttribute("disabled", true);
          this.cc_alert.setAttribute("disabled", true);
        }
        break;
      case "slider.freq":
        this.cc_alert.setAttribute("value", value);
        this.cc_alert.value = value;
        
        if (value == 0) {
          this.cc_alert_disable.removeAttribute("checked");
          this.cc_alert.setAttribute("disabled", true);
        } else {
          this.cc_alert_disable.setAttribute("checked", true);
          this.cc_alert.removeAttribute("disabled");
        }
          
        break;
      case "locid":
        this.locid.setAttribute("value", value);
        this.locid.value = value;
        break;
      case "celcius":
        this.metric.selectedItem = (value) ? this.temp_c : this.temp_f;
        break;
      case "display.bar":
        try {
          this.barinput.selectedItem = document.getElementById(value);
        }
        catch (e) {}
        break;
      case "display.nextnode":
        try {
          this.nextnode.selectedItem = document.getElementById(value);
        }
        catch (e) {}
        break;
      case "display.insertafter":
        try {
          this.align.selectedItem = document.getElementById(value ? "insertright" : "insertleft");
        }
        catch (e) {}
        break;
      case "forecast.days":
        this.dayf_days.setAttribute("value", value);
        this.dayf_days.value = value;
        if (value == 0) {
          this.dayf_show_days.setAttribute("disabled", true);
          this.dayf_show_nights.setAttribute("disabled", true);
          this.dayf_show_img.setAttribute("disabled", true);
          this.dayf_show_temp.setAttribute("disabled", true);
        }
        break;
      case "current.mode":
        switch (value) {
          case 2:
            this.cc_temp.setAttribute("checked", "true");
            this.cc_img.setAttribute("checked", "true");
            break;
          case 1:
            this.cc_temp.setAttribute("checked", "true");
            this.cc_img.removeAttribute("checked");
            break;
          case 0:
          default:
            this.cc_img.setAttribute("checked", "true");
            this.cc_temp.removeAttribute("checked");
            break;  
        }
        break;
      case "forecast.parts":
        switch (value) {
          case 2:
            this.dayf_show_days.setAttribute("checked", "true");
            this.dayf_show_nights.setAttribute("checked", "true");
            break;
          case 1:
            this.dayf_show_days.removeAttribute("checked");
            this.dayf_show_nights.setAttribute("checked", "true");
            break;
          case 0:
          default:
            this.dayf_show_days.setAttribute("checked", "true");
            this.dayf_show_nights.removeAttribute("checked");
            break;
        }
        break;
      case "forecast.mode":
        switch(value) {
          case 2:
            this.dayf_show_temp.setAttribute("checked", "true");
            this.dayf_show_img.setAttribute("checked", "true");
            break;
          case 1:
            this.dayf_show_img.removeAttribute("checked");
            this.dayf_show_temp.setAttribute("checked", "true");
            break;
          case 0:
          default:
            this.dayf_show_img.setAttribute("checked", "true");
            this.dayf_show_temp.removeAttribute("checked");
            break;
        }
        break;
      case "profile":
        this.profile_current.setAttribute("value", value);
        this.profile_current.value = value;
        break;
    }
  },
  
  getPrefs: function(pref_list)
  {
    var prefs = new Object();
    
    for (var value in pref_list)
      prefs[value] = this.getPref(value);
      
    return prefs;
  },
  
  updatePrefs: function()
  {
    this.wfPrefs = new Object();
    this.wfPrefs["locid"] = this.wfBranch.getCharPref("locid");
    this.wfPrefs["celcius"] = this.wfBranch.getBoolPref("celcius");
    this.wfPrefs["profile"] = this.wfBranch.getCharPref("profile");
    this.wfPrefs["display.bar"] = this.wfBranch.getCharPref("display.bar");
    this.wfPrefs["display.nextnode"] = this.wfBranch.getCharPref("display.nextnode");
    this.wfPrefs["display.insertafter"] = this.wfBranch.getBoolPref("display.insertafter"); 
    this.wfPrefs["current.display"] = this.wfBranch.getBoolPref("current.display");
    this.wfPrefs["current.mode"] = this.wfBranch.getIntPref("current.mode");
    this.wfPrefs["forecast.days"] = this.wfBranch.getIntPref("forecast.days");
    this.wfPrefs["forecast.mode"] = this.wfBranch.getIntPref("forecast.mode");
    this.wfPrefs["forecast.parts"] = this.wfBranch.getIntPref("forecast.parts");
    this.wfPrefs["slider.freq"] = this.wfBranch.getIntPref("slider.freq");
  },
  
  getPref: function(pref)
  {
    var value = null;
    switch(pref) {
      case "current.display":
        if (!this.cc_display.getAttribute("checked"))
          return false;
        else if (!this.cc_img.getAttribute("checked") && !this.cc_temp.getAttribute("checked"))
          return false;
        else
          return true;
      case "slider.freq":
        return (this.cc_alert_disable.getAttribute("checked") ? this.cc_alert.value : 0);
      case "locid":
        return this.locid.value;
      case "celcius":
        return (this.metric.selectedItem == this.temp_c);
      case "display.bar":
        return this.barinput.selectedItem.getAttribute("id");
      case "display.nextnode":
        return this.nextnode.selectedItem.getAttribute("id");
      case "display.insertafter":
        return (this.align.selectedItem.getAttribute("id") == "insertright");
      case "forecast.days":
        value = this.dayf_days.value;
        
        if ((!this.dayf_show_temp.getAttribute("checked") && !this.dayf_show_img.getAttribute("checked")) ||
            (!this.dayf_show_days.getAttribute("checked") && !this.dayf_show_nights.getAttribute("checked")))
            value = 0;
            
        return (Number(value)) ? value : 0;
      case "current.mode":
        var cc_temp = this.cc_temp.getAttribute("checked") ? true : false;
        var cc_img = this.cc_img.getAttribute("checked") ? true : false;
        
        if (cc_temp && cc_img)
          value = 2;
        else if (cc_temp)
          value = 1;
        else
          value = 0;
          
        return value;
        
      case "forecast.parts":
        var dayf_days = this.dayf_show_days.getAttribute("checked") ? true : false;
        var dayf_nights = this.dayf_show_nights.getAttribute("checked") ? true : false;
        
        if (dayf_days && dayf_nights)
          value = 2;
        else if (dayf_nights)
          value = 1;
        else
          value = 0;
          
        return value;
        
      case "forecast.mode":
        var dayf_temp = this.dayf_show_temp.getAttribute("checked") ? true : false;
        var dayf_img = this.dayf_show_img.getAttribute("checked") ? true : false;
        
        if (dayf_temp && dayf_img)
          value = 2;
        else if (dayf_temp)
          value = 1;
        else
          value = 0;
        
        return value;
        
      case "profile":
        return this.profile_current.value;
    }
  },
  
  profContext: function()
  {
    // builds the context menu in the profile list box
    if (this.profile_list.selectedItem.getAttribute("label") == this.profile_current.value) {
      document.getElementById("wf_select").setAttribute("hidden", true);
      document.getElementById("wf_remove").setAttribute("hidden", true);
    }
    else {
      document.getElementById("wf_select").removeAttribute("hidden");
      document.getElementById("wf_remove").removeAttribute("hidden");
    }
  },
  
  populateProfiles: function()
  {
    var count = {};
    var keys = this.wfProfile.getNames(count);
    var item;
    
    for (var x = 0; x < count.value; x++)
      this.createProfileItem(keys[x]);

    var listitems = this.profile_list.childNodes;
    
    for (x = 0; x < listitems.length; x++)
      if (listitems[x].getAttribute("label") == this.wfPrefs["profile"])
        this.profile_list.selectItem(listitems[x]);
    
  },
  
  createProfileItem: function(name)
  {
    var item = this.profile_list.appendItem(name);
    item.setAttribute("ondblclick", "if (event.button == 0) wfOptions.selectProfile();");
  },
  
  updateProfButtons: function()
  {
    this.button_rename.removeAttribute("disabled");
    if (this.profile_list.selectedItem) {
      if (this.profile_list.selectedItem.getAttribute("label") == this.profile_current.value) {
        this.button_remove.setAttribute("disabled", true);
        this.button_select.setAttribute("disabled", true);
      }
      else {
        this.button_remove.removeAttribute("disabled");
        this.button_select.removeAttribute("disabled"); 
      }
    }
    else
      this.button_remove.setAttribute("disabled", true);
  },
  
  updateButtons: function(event)
  {
    // don't disable the accept button if its the accept button
    if (event) {
      if (event.originalTarget.getAttribute("dlgtype") == "accept")
        return;
      if (event.originalTarget != document.getElementById("wf_prof_delete"))
      this.profile_inuse.setAttribute("hidden", true);
    }
    
    var newPrefs = this.getPrefs(this.wfPrefs);
    for (var value in newPrefs) {
      if (newPrefs[value] != this.wfPrefs[value]) {
        this.button_apply.removeAttribute("disabled");
        this.button_accept.removeAttribute("disabled");
        return;
      }
    }

    this.button_apply.setAttribute("disabled", true);
    this.button_accept.setAttribute("disabled", true);
  },
  
  selectProfile: function()
  {
    var profile = this.profile_list.selectedItem.getAttribute("label");
    this.wfBranch.setCharPref("profile", profile);
    this.updateProfButtons();
  },
  
  removeProfile: function()
  {
    var profile = this.profile_list.selectedItem;
    
    // check to see if its the current profile
    if (profile.getAttribute("label") == this.wfBranch.getCharPref("profile")) {
      this.profile_inuse.removeAttribute("hidden");
      return;
    }
    
    // tell the profile service to remove it
    this.wfProfile.deleteProfile(profile.getAttribute("label"));
    
    // then remove it from the list
    profile.parentNode.removeChild(profile);

    // and select the first list item
    this.profile_list.selectItem(this.profile_list.firstChild);
  },
  
  topWindow: function()
  {
    //get a navigator window
    var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
    var windowManagerInterface = windowManager.QueryInterface( Components.interfaces.nsIWindowMediator);
    var topWindow = windowManagerInterface.getMostRecentWindow("navigator:browser");
    if (!topWindow)
      topWindow = window.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", "about:blank", null, null);

    return topWindow;
  },
  
  populateBars: function()
  {      
   // Creates the menuitems for the toolbar selector.
    var topWindow = this.topWindow();
    var toolbars = topWindow.document.getElementsByTagName("toolbar");
    var statusbars = topWindow.document.getElementsByTagName("statusbar");
    var pnode = document.getElementById("wfbarpopup");
    var bar_id, newitem;
          
    toolbars = concat(toolbars, statusbars);
    for (var x = 0; x < toolbars.length; x++) {
      bar_id = toolbars[x].getAttribute("id");
      //do not include find toolbar
      if (bar_id == "FindToolbar")
        continue;       
      newitem = document.createElement("menuitem");
      newitem.setAttribute("id", bar_id);
      newitem.setAttribute("label", bar_id);
      pnode.appendChild(newitem);
    }
  },
  
  populateNodes: function()
  {
    // Creates the menuitems for the nextnode selector.
    var topWindow = this.topWindow();       
    var pnode = document.getElementById("neighborinputpopup");
    var nodes = pnode.childNodes;
    var bar_id = document.getElementById("barinput").selectedItem.getAttribute("id");
    var node_id, newitem, x;
  
    // Remove all the menuitems currently in the popup.
    for (x = 0; x < nodes.length; x++)
      pnode.removeChild(nodes[x]);
  
    nodes = topWindow.document.getElementById(bar_id).childNodes;
    for (x = 0; x < nodes.length; x++) {
      node_id = nodes[x].getAttribute("id");
  
      // Don't include the wf_deck or generated content in the list.
      if (node_id == "wf_deck" || nodes[x].tagName == "toolbarspacer" || 
        nodes[x].tagName == "toolbarseparator" || nodes[x].tagName == "toolbarspring") 
        continue;
  
      newitem = document.createElement("menuitem");
      newitem.setAttribute("id", node_id);
      newitem.setAttribute("label", node_id);
      pnode.appendChild(newitem);
    }
  
    // And finally select the first one in the list.
    pnode.parentNode.selectedItem = pnode.firstChild;
  },
  
  open: function(aEvent)
  {
    var topWindow = this.topWindow();
    var browser = topWindow.document.getElementById("content");
    var URL = aEvent.currentTarget.getAttribute("url") + "?prod=" + this.wfUpdate.productCode + "&par=" + this.wfUpdate.partnerId;
    browser.loadURI(URL);
  },
  
  accept: function() {
    var newPrefs = this.getPrefs(this.wfPrefs);

    if (newPrefs["locid"] != this.wfPrefs["locid"])
      this.wfBranch.setCharPref("locid", newPrefs["locid"]);
    if (newPrefs["celcius"] != this.wfPrefs["celcius"])
      this.wfBranch.setBoolPref("celcius", newPrefs["celcius"]);  
    if (newPrefs["display.bar"] != this.wfPrefs["display.bar"])
      this.wfBranch.setCharPref("display.bar", newPrefs["display.bar"]);
    if (newPrefs["display.nextnode"] != this.wfPrefs["display.nextnode"])
      this.wfBranch.setCharPref("display.nextnode", newPrefs["display.nextnode"]);  
    if (newPrefs["display.insertafter"] != this.wfPrefs["display.insertafter"])
      this.wfBranch.setBoolPref("display.insertafter", newPrefs["display.insertafter"]);
    if (newPrefs["current.display"] != this.wfPrefs["current.display"])
      this.wfBranch.setBoolPref("current.display", newPrefs["current.display"]);  
    if (newPrefs["forecast.days"] != this.wfPrefs["forecast.days"])
      this.wfBranch.setIntPref("forecast.days", newPrefs["forecast.days"]);
    if (newPrefs["slider.freq"] != this.wfPrefs["slider.freq"])
      this.wfBranch.setIntPref("slider.freq", newPrefs["slider.freq"]);
    if (newPrefs["current.mode"] != this.wfPrefs["current.mode"])
      this.wfBranch.setIntPref("current.mode", newPrefs["current.mode"]);
    if (newPrefs["forecast.mode"] != this.wfPrefs["forecast.mode"])
      this.wfBranch.setIntPref("forecast.mode", newPrefs["forecast.mode"]);
    if (newPrefs["forecast.parts"] != this.wfPrefs["forecast.parts"])
      this.wfBranch.setIntPref("forecast.parts", newPrefs["forecast.parts"]);
    
    this.wfPrefs = newPrefs;
  },
  
  apply: function()
  {
    this.accept();
    this.button_apply.setAttribute("disabled", true);
  },
  
  checkModes: function(type)
  {
    switch (type) {
      case "current":
        this.cc_img.removeAttribute("disabled");
        this.cc_temp.removeAttribute("disabled");
        this.cc_alert.removeAttribute("disabled");
        this.cc_alert_disable.removeAttribute("disabled");
        
        if (!this.getPref("current.display")) {
          this.cc_img.setAttribute("disabled", true);
          this.cc_temp.setAttribute("disabled", true);
          this.cc_alert.setAttribute("disabled", true);
          this.cc_alert_disable.setAttribute("disabled", true);
        }
        this.checkModes("alert");
        break;
      case "forecast":
        this.dayf_show_temp.removeAttribute("disabled");
        this.dayf_show_img.removeAttribute("disabled");
        this.dayf_show_days.removeAttribute("disabled");
        this.dayf_show_nights.removeAttribute("disabled");
        
        if (this.getPref("forecast.days") == "0" || this.getPref("forecast.days") == "") {
          this.dayf_show_temp.setAttribute("disabled", true);
          this.dayf_show_img.setAttribute("disabled", true);
          this.dayf_show_days.setAttribute("disabled", true);
          this.dayf_show_nights.setAttribute("disabled", true);
        }
        break;
      case "alert":
        this.cc_alert.removeAttribute("disabled");
        
        if (!this.cc_alert_disable.getAttribute("checked"))
          this.cc_alert.setAttribute("disabled", true);
 
        break;
    }
   
  },
  
  ////////////////////
  // observe
  
  observe: function(aSubject, aTopic, aData) {
    var pref;
    switch (aTopic) {
      case "forecastfox:profile:new":
        this.createProfileItem(aData);
        break;
      case "forecastfox:profile:end":
        this.updatePrefs();
        this.syncPrefs(this.wfPrefs);
        this.syncPref("display.bar", this.wfPrefs["display.bar"]);
        this.populateNodes();
        this.syncPref("display.nextnode", this.wfPrefs["display.nextnode"]);
        break;
    }
  }
};

function concat(c1, c2) 
{
  // Concats too collections into an array.
  var c3 = new Array(c1.length + c2.length);
  var x,y = 0;
    
  for (x = 0; x < c1.length; x++)
    c3[y++] = c1[x];
        
  for (x = 0; x < c2.length; x++) 
    c3[y++] = c2[x];
        
  return c3;
}
