/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ForecastFox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Jon Stritar <jstritar@MIT.EDU>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
// global variables
var CURRENT_DEFAULT  = null;
var FORECAST_DEFAULT = null;
var CURRENT_L_DEFAULT = null;
var FORECAST_L_DEFAULT = null;


var wf_tooltip = null;

// functions

function  wfTooltipLoad() 
{
  var strings = document.getElementById("wf_bundle");
  
  CURRENT_DEFAULT = strings.getString("wf.cc.tooltip");
  FORECAST_DEFAULT = strings.getString("wf.dayf.tooltip");
  CURRENT_L_DEFAULT = strings.getString("wf.cc.label"); 
  FORECAST_L_DEFAULT =  strings.getString("wf.dayf.label");
  
  wf_tooltip = new wfTooltip();
}

function wfTooltipUnload() 
{
  wf_tooltip.unload();
  wf_tooltip = null;
  
  CURRENT_DEFAULT  = null;
  FORECAST_DEFAULT = null;
  CURRENT_L_DEFAULT = null;
  FORECAST_L_DEFAULT = null; 
}


function wfTooltip() 
{
  var prefService = "@mozilla.org/preferences-service;1";
  prefService = Components.classes[prefService].getService(Components.interfaces.nsIPrefService);

  this.wfUpdate = Components.classes["@mozdev.org/forecastfox/update;1"]
                   .getService(Components.interfaces.nsIForecastfoxUpdate);
                   
  // set up all the internal object variables
  this.wfBranch         = prefService.getBranch("forecastfox.");
  this.bundle       = document.getElementById("wf_bundle");
  this.current_t        = document.getElementById("currentinput");
  this.forecast_t         = document.getElementById("forecastinput");
  this.current_l        = document.getElementById("currentlabel");
  this.forecast_l       = document.getElementById("forecastlabel");
  this.current_i        = document.getElementById("wf_ccimage");
  this.forecast_i       = document.getElementById("wf_forecastimage"); 
  this.accept           = document.documentElement.getButton("accept");
  this.apply          = document.documentElement.getButton("help");
  if (this.wfBranch.prefHasUserValue("current.tooltip"))
    this.current_tooltip    = this.wfBranch.getCharPref("current.tooltip");
  else
    this.current_tooltip    = this.bundle.getString("wf.cc.tooltip");
  if (this.wfBranch.prefHasUserValue("forecast.tooltip"))      
    this.forecast_tooltip   = this.wfBranch.getCharPref("forecast.tooltip");
  else
    this.forecast_tooltip   = this.bundle.getString("wf.dayf.tooltip");
  if (this.wfBranch.prefHasUserValue("current.label"))  
    this.current_label      = this.wfBranch.getCharPref("current.label");
  else
    this.current_label     = this.bundle.getString("wf.cc.label");  
  if (this.wfBranch.prefHasUserValue("forecast.label"))  
    this.forecast_label     = this.wfBranch.getCharPref("forecast.label");
  else
    this.forecast_label     = this.bundle.getString("wf.dayf.label");   
  this.current_icon         = this.wfBranch.getBoolPref("current.tipicon");
  this.forecast_icon        = this.wfBranch.getBoolPref("forecast.tipicon");
  this.currentrestore_t   = document.getElementById("currentrestore-t");
  this.currentrestore_l   = document.getElementById("currentrestore-l");
  this.forecastrestore_t  = document.getElementById("forecastrestore-t");
  this.forecastrestore_l  = document.getElementById("forecastrestore-l");
 
  this.tree           = document.getElementById("tt_tree");
  this.clipBoardHelper    = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
                                        .getService(Components.interfaces.nsIClipboardHelper);
                                       
  this.cpy_var          = document.getElementById("cpy_var");
  this.cpy_des          = document.getElementById("cpy_des");
  this.cpy_cat          = document.getElementById("cpy_cat");
  
  // set up the initial box
  this.load();

  // return reference of self
  return this;
}

wfTooltip.prototype = {

  load: function() 
  {

    // then set up the dialog box
    this.apply.label = document.getElementById("extra1-label").getAttribute("label");
    this.apply.removeAttribute("accesskey");
    this.current_t.setAttribute("value", this.current_tooltip);
    this.forecast_t.setAttribute("value", this.forecast_tooltip);
    this.current_l.setAttribute("value", this.current_label);
    this.forecast_l.setAttribute("value", this.forecast_label);
    if (this.current_icon)
      this.current_i.setAttribute("checked", "true");
    if (this.forecast_icon)
      this.forecast_i.setAttribute("checked", "true");    
    this.populateTree();

    this.onChange();
  },

  unload: function() 
  {

    // make sure we don't leak
    this.wfUpdate         = null;
    this.wfBranch       = null;
    this.current_t      = null;
    this.forecast_t     = null;
    this.current_l      = null;
    this.forecast_l     = null;
    this.apply        = null;
    this.current_tooltip  = null;
    this.forecast_tooltip = null;
    this.current_label    = null;
    this.forecast_label   = null;
    this.current_icon     = null;
    this.current_i        = null;
    this.forecast_icon    = null;
    this.forecast_i       = null;   
    this.bundle           = null; 
  },

  acceptChanges: function() 
  {

    // Saves any changes in the data
    if (this.current_t.value != this.current_tooltip) {
      if (this.current_t.value == CURRENT_DEFAULT)
        this.wfBranch.clearUserPref("current.tooltip");
      else
        this.wfBranch.setCharPref("current.tooltip", this.current_t.value);
    }
    if (this.forecast_t.value != this.forecast_tooltip) {
      if (this.forecast_t.value == FORECAST_DEFAULT)
        this.wfBranch.clearUserPref("forecast.tooltip");
      else
        this.wfBranch.setCharPref("forecast.tooltip", this.forecast_t.value);
    }
    if (this.current_l.value != this.current_label) {
      if (this.current_l.value == CURRENT_L_DEFAULT)
        this.wfBranch.clearUserPref("current.label");
      else
        this.wfBranch.setCharPref("current.label", this.current_l.value);
    }
    if (this.forecast_l.value != this.forecast_label) {
      if (this.forecast_l.value == FORECAST_L_DEFAULT)
        this.wfBranch.clearUserPref("forecast.label");
      else
        this.wfBranch.setCharPref("forecast.label", this.forecast_l.value);
    }
    
    var cc_ic = this.current_i.getAttribute("checked") ? true : false;
    var forecast_ic = this.forecast_i.getAttribute("checked") ? true : false;
    if (cc_ic != this.current_icon)
      this.wfBranch.setBoolPref("current.tipicon", cc_ic);
    if (forecast_ic != this.forecast_icon)
      this.wfBranch.setBoolPref("forecast.tipicon", forecast_ic);    
  },

  applyChanges: function() 
  {
  
    // Saves changes in the data without closing the window
    this.apply.setAttribute("disabled", true);
    this.acceptChanges();

    this.current_tooltip  = this.current_t.value;
    this.forecast_tooltip = this.forecast_t.value;
    this.current_label    = this.current_l.value;
    this.forecast_label   = this.forecast_l.value;
  },

  copy: function(type, event) 
  {
    if (this.tree.currentIndex == -1)
      return;

    var item = this.tree.contentView.getItemAtIndex(this.tree.currentIndex);
    var cells = item.getElementsByTagName("treecell");
    var copy_string;

    switch (type) {
      case "category" :
      case "variable" :
        copy_string = cells[0].getAttribute("label");
        break;
      case "description" :
        copy_string = cells[1].getAttribute("label");
        break;
    }

    this.clipBoardHelper.copyString(copy_string);
  },
  
  openPopup: function() {
    // If its not a variable/description entry don't show the popup.
    if (this.tree.contentView.getItemAtIndex(this.tree.currentIndex).getElementsByTagName("treecell").length != 2) {
      this.cpy_var.setAttribute("hidden", true);
      this.cpy_des.setAttribute("hidden", true);
      this.cpy_cat.removeAttribute("hidden");
    }
    else {
      this.cpy_var.removeAttribute("hidden");
      this.cpy_des.removeAttribute("hidden");
      this.cpy_cat.setAttribute("hidden", true);
    }
  },
  
  restoreDefault: function(type, tooltip) 
  {
    // Puts the default values back in the tooltip textboxes
    switch(type) {
      case 'current':
        if (tooltip)
          this.current_t.value = CURRENT_DEFAULT;
        else
          this.current_l.value = CURRENT_L_DEFAULT;
        break;
      case 'forecast':
        if (tooltip)
          this.forecast_t.value = FORECAST_DEFAULT;
        else
          this.forecast_l.value = FORECAST_L_DEFAULT;
        break;
    }

    this.onChange();
  },

  onChange: function() 
  {
    var cc_ic = this.current_i.getAttribute("checked") ? true : false;
    var forecast_ic = this.forecast_i.getAttribute("checked") ? true : false;
      
    // Disables/enables accept and apply based on if there are changes
    if ((this.current_t.value != this.current_tooltip) ||
        (this.forecast_t.value != this.forecast_tooltip) ||
        (this.current_l.value != this.current_label) ||
        (this.forecast_l.value != this.forecast_label) ||
        (this.current_icon != cc_ic) ||
        (this.forecast_icon != forecast_ic)){

      this.apply.removeAttribute("disabled");
      this.accept.removeAttribute("disabled");
    }
    else {
      this.apply.setAttribute("disabled", true);
      this.accept.setAttribute("disabled", true);
    }

    if (this.current_t.value == CURRENT_DEFAULT)
      this.currentrestore_t.setAttribute("disabled", true);
    else
      this.currentrestore_t.removeAttribute("disabled");

    if (this.current_l.value ==CURRENT_L_DEFAULT)
      this.currentrestore_l.setAttribute("disabled", true);
    else
      this.currentrestore_l.removeAttribute("disabled");

    if (this.forecast_t.value == FORECAST_DEFAULT)
      this.forecastrestore_t.setAttribute("disabled", true);
    else
      this.forecastrestore_t.removeAttribute("disabled");

    if (this.forecast_l.value == FORECAST_L_DEFAULT)
      this.forecastrestore_l.setAttribute("disabled", true);
    else
      this.forecastrestore_l.removeAttribute("disabled");
  },

  populateTree: function() 
  {
    var strings = document.getElementById("wf_bundle");
    var a_current = [["[lsup]", strings.getString("wf.tt.lsup")],
                     ["[obst]", strings.getString("wf.tt.obst")],
                     ["[tmp]", strings.getString("wf.tt.tmp")],
                     ["[flik]", strings.getString("wf.tt.flik")],
                     ["[barr]", strings.getString("wf.tt.barr")],
                     ["[bard]", strings.getString("wf.tt.bard")],
                     ["[hmid]", strings.getString("wf.tt.hmid")],
                     ["[vis]", strings.getString("wf.tt.vis")],
                     ["[uvi]", strings.getString("wf.tt.uvi")],
                     ["[uvt]", strings.getString("wf.tt.uvt")],
                     ["[dewp]", strings.getString("wf.tt.dewp")]];

    var a_forecast = [["[part]", strings.getString("wf.tt.part")],
                      ["[day]", strings.getString("wf.tt.day")],
                      ["[sday]", strings.getString("wf.tt.sday")],
                      ["[date]", strings.getString("wf.tt.date")],
                      ["[lsup]", strings.getString("wf.tt.lsup")],
                      ["[tmp]", strings.getString("wf.tt.tmp2")],
                      ["[hi]", strings.getString("wf.tt.hi")],
                      ["[low]", strings.getString("wf.tt.low")],
                      ["[ppcp]", strings.getString("wf.tt.ppcp")],
                      ["[hmid]", strings.getString("wf.tt.hmid")]];

    var a_common = [["/n", strings.getString("wf.tt.nl")],
                    ["[prof]", strings.getString("wf.tt.prof")],
                    ["[dnam]", strings.getString("wf.tt.dnam")],
                    ["[lat]", strings.getString("wf.tt.lat")],
                    ["[lon]", strings.getString("wf.tt.lon")],
                    ["[zone]", strings.getString("wf.tt.zone")],
                    ["[icon]", strings.getString("wf.tt.icon")],
                    ["[tm]", strings.getString("wf.tt.tm")],
                    ["[sunr]", strings.getString("wf.tt.sunr")],
                    ["[suns]", strings.getString("wf.tt.suns")],
                    ["[t]", strings.getString("wf.tt.t")],
                    ["[winds]", strings.getString("wf.tt.winds")],
                    ["[windgust]", strings.getString("wf.tt.windgust")],
                    ["[windd]", strings.getString("wf.tt.windd")],
                    ["[windt]", strings.getString("wf.tt.windt")]];

    var tree_current  = document.getElementById("tree_current");
    var tree_forecast = document.getElementById("tree_forecast");
    var tree_common   = document.getElementById("tree_common");
    var x;

    // Now add all the data to the tree.
    for (x = 0; x < a_current.length; x++)
      this.addTreeRow( tree_current, a_current[x] );

    for (x = 0; x < a_forecast.length; x++)
      this.addTreeRow( tree_forecast, a_forecast[x] );

    for (x = 0; x < a_common.length; x++)
      this.addTreeRow( tree_common, a_common[x] );
  },
  
  openLink: function(aEvent, aLoc)
  {
    var url = aEvent.currentTarget.getAttribute("url");
    if (aLoc) {
      var locid = this.mBranch.getCharPref("locid");
      url = url + locid + "?prod=" + this.mUpdate.productCode + "&par=" + this.mUpdate.partnerId;
    }
    else
      url = url + "?prod=" + this.mUpdate.productCode + "&par=" + this.mUpdate.partnerId;
      
    if (aEvent.target.parentNode == document.getElementById("wf_forecast"))
      url = url + "#tenday";

    openUILink(url, aEvent);
  },

  addTreeRow: function( aParent, aData ) 
  {
    var item = document.createElement("treeitem");
    var node = document.createElement("treerow");
    var cell;

    for (var value in aData) {
      // Create the cell
      cell = document.createElement("treecell");
      cell.setAttribute("label", aData[value]);

      // And add it onto the row.
      node.appendChild(cell);
    }

    // When done add the node onto the item, then the item onto the aParent.
    item.appendChild(node);
    aParent.appendChild(item);
  },
  
  open: function(aEvent)
  {
    var topWindow = this.topWindow();
    var browser = topWindow.document.getElementById("content");
    var URL = aEvent.currentTarget.getAttribute("url") + "?prod=" + this.wfUpdate.productCode + "&par=" + this.wfUpdate.partnerId;
    browser.loadURI(URL);
  },
  
  topWindow: function()
  {
    //get a navigator window
    var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
    var windowManagerInterface = windowManager.QueryInterface( Components.interfaces.nsIWindowMediator);
    var topWindow = windowManagerInterface.getMostRecentWindow("navigator:browser");
    if (!topWindow)
      topWindow = window.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", "about:blank", null, null);

    return topWindow;
  }
};


function concat(c1, c2) 
{
    // Concats too collections into an array.
    var c3 = new Array(c1.length + c2.length);
    var x,y = 0;

    for (x = 0; x < c1.length; x++)
        c3[y++] = c1[x];

    for (x = 0; x < c2.length; x++)
        c3[y++] = c2[x];

    return c3;
}
